@@ -25,10 +25,13 @@
 
 /******************************************************************************/
 
-(function() {
+(( ) => {
 
 /******************************************************************************/
 
+// TODO: fix the inconsistencies re. realm vs. filter source which have
+//       accumulated over time.
+
 const messaging = vAPI.messaging;
 const logger = self.logger = { ownerId: Date.now() };
 const logDate = new Date();
@@ -41,8 +44,10 @@ let filteredLoggerEntryVoidedCount = 0;
 let popupLoggerBox;
 let popupLoggerTooltips;
 let activeTabId = 0;
+let filterAuthorMode = false;
 let selectedTabId = 0;
 let netInspectorPaused = false;
+let cnameOfEnabled = false;
 
 /******************************************************************************/
 
@@ -64,7 +69,7 @@ const tabIdFromAttribute = function(elem) {
 
 // Current design allows for only one modal DOM-based dialog at any given time.
 //
-const modalDialog = (function() {
+const modalDialog = (( ) => {
     const overlay = uDom.nodeFromId('modalOverlay');
     const container = overlay.querySelector(
         ':scope > div > div:nth-of-type(1)'
@@ -155,9 +160,12 @@ const regexFromURLFilteringResult = function(result) {
 
 const nodeFromURL = function(parent, url, re) {
     const fragment = document.createDocumentFragment();
-    if ( re instanceof RegExp === false ) {
+    if ( re === undefined ) {
         fragment.textContent = url;
     } else {
+        if ( typeof re === 'string' ) {
+            re = new RegExp(re.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'), 'g');
+        }
         const matches = re.exec(url);
         if ( matches === null || matches[0].length === 0 ) {
             fragment.textContent = url;
@@ -209,6 +217,10 @@ const LogEntry = function(details) {
             this[prop] = details[prop];
         }
     }
+    this.type = details.stype || details.type;
+    if ( details.aliasURL !== undefined ) {
+        this.aliased = true;
+    }
     if ( this.tabDomain === '' ) {
         this.tabDomain = this.tabHostname || '';
     }
@@ -220,11 +232,13 @@ const LogEntry = function(details) {
     }
 };
 LogEntry.prototype = {
+    aliased: false,
     dead: false,
     docDomain: '',
     docHostname: '',
     domain: '',
     filter: undefined,
+    id: '',
     realm: '',
     tabDomain: '',
     tabHostname: '',
@@ -291,7 +305,13 @@ const processLoggerEntries = function(response) {
             if ( autoDeleteVoidedRows ) { continue; }
             parsed.voided = true;
         }
-        if ( parsed.type === 'main_frame' ) {
+        if (
+            parsed.type === 'main_frame' &&
+            parsed.aliased === false && (
+                parsed.filter === undefined ||
+                parsed.filter.source !== 'redirect'
+            )
+        ) {
             const separator = createLogSeparator(parsed, unboxed.url);
             loggerEntries.unshift(separator);
             if ( rowFilterer.filterOne(separator) ) {
@@ -301,6 +321,10 @@ const processLoggerEntries = function(response) {
                 }
             }
         }
+        if ( cnameOfEnabled === false && parsed.aliased ) {
+            uDom.nodeFromId('filterExprCnameOf').style.display = '';
+            cnameOfEnabled = true;
+        }
         loggerEntries.unshift(parsed);
         if ( rowFilterer.filterOne(parsed) ) {
             filteredLoggerEntries.unshift(parsed);
@@ -320,6 +344,11 @@ const processLoggerEntries = function(response) {
 /******************************************************************************/
 
 const parseLogEntry = function(details) {
+    // Patch realm until changed all over codebase to make this unecessary
+    if ( details.realm === 'cosmetic' ) {
+        details.realm = 'extended';
+    }
+
     const entry = new LogEntry(details);
 
     // Assemble the text content, i.e. the pre-built string which will be
@@ -362,7 +391,7 @@ const parseLogEntry = function(details) {
     // Cell 3
     textContent.push(normalizeToStr(entry.docHostname));
 
-    // Cell 4
+    // Cell 4: partyness
     if (
         entry.realm === 'network' &&
         typeof entry.domain === 'string' &&
@@ -398,6 +427,13 @@ const parseLogEntry = function(details) {
     // Cell 6
     textContent.push(normalizeToStr(details.url));
 
+    // Hidden cells -- useful for row-filtering purpose
+
+    // Cell 7
+    if ( entry.aliased ) {
+        textContent.push(`aliasURL=${details.aliasURL}`);
+    }
+
     entry.textContent = textContent.join('\t');
     return entry;
 };
@@ -413,6 +449,8 @@ const viewPort = (( ) => {
     const vwLogEntryTemplate = document.querySelector('#logEntryTemplate > div');
     const vwEntries = [];
 
+    const detailableRealms = new Set([ 'network', 'extended' ]);
+
     let vwHeight = 0;
     let lineHeight = 0;
     let wholeHeight = 0;
@@ -618,7 +656,6 @@ const viewPort = (( ) => {
         const divcl = div.classList;
         let span;
 
-
         // Realm
         if ( details.realm !== undefined ) {
             divcl.add(details.realm + 'Realm');
@@ -645,7 +682,7 @@ const viewPort = (( ) => {
             return div;
         }
 
-        if ( details.realm === 'network' || details.realm === 'cosmetic' ) {
+        if ( detailableRealms.has(details.realm) ) {
             divcl.add('canDetails');
         }
 
@@ -658,11 +695,13 @@ const viewPort = (( ) => {
             }
             if ( filteringType === 'static' ) {
                 divcl.add('canLookup');
-                div.setAttribute('data-filter', filter.compiled);
-            } else if ( filteringType === 'cosmetic' ) {
-                divcl.add('canLookup');
+            } else if ( details.realm === 'extended' ) {
+                divcl.toggle('canLookup', /^#@?#/.test(filter.raw));
                 divcl.toggle('isException', filter.raw.startsWith('#@#'));
             }
+            if ( filter.modifier === true ) {
+                div.setAttribute('data-modifier', '');
+            }
         }
         span = div.children[1];
         if ( renderFilterToSpan(span, cells[1]) === false ) {
@@ -713,7 +752,7 @@ const viewPort = (( ) => {
         span.textContent = cells[5];
 
         // URL
-        let re = null;
+        let re;
         if ( filteringType === 'static' ) {
             re = new RegExp(filter.regex, 'gi');
         } else if ( filteringType === 'dynamicUrl' ) {
@@ -721,6 +760,14 @@ const viewPort = (( ) => {
         }
         nodeFromURL(div.children[6], cells[6], re);
 
+        // Alias URL (CNAME, etc.)
+        if ( cells.length > 7 ) {
+            const pos = details.textContent.lastIndexOf('\taliasURL=');
+            if ( pos !== -1 ) {
+                div.setAttribute('data-aliasid', details.id);
+            }
+        }
+
         return div;
     };
 
@@ -949,6 +996,8 @@ const onLogBufferRead = function(response) {
         allTabIdsToken = response.tabIdsToken;
     }
 
+    filterAuthorMode = response.filterAuthorMode === true;
+
     if ( activeTabIdChanged ) {
         pageSelectorFromURLHash();
     }
@@ -975,7 +1024,7 @@ const onLogBufferRead = function(response) {
 const readLogBuffer = (( ) => {
     let timer;
 
-    const readLogBufferNow = function() {
+    const readLogBufferNow = async function() {
         if ( logger.ownerId === undefined ) { return; }
 
         const msg = {
@@ -1002,11 +1051,11 @@ const readLogBuffer = (( ) => {
             msg.popupLoggerBoxChanged = true;
         }
 
-        vAPI.messaging.send('loggerUI', msg, response => {
-            timer = undefined;
-            onLogBufferRead(response);
-            readLogBufferLater();
-        });
+        const response = await vAPI.messaging.send('loggerUI', msg);
+
+        timer = undefined;
+        onLogBufferRead(response);
+        readLogBufferLater();
     };
 
     const readLogBufferLater = function() {
@@ -1078,24 +1127,23 @@ const reloadTab = function(ev) {
     messaging.send('loggerUI', {
         what: 'reloadTab',
         tabId: tabId,
-        bypassCache: ev && (ev.ctrlKey || ev.metaKey || ev.shiftKey)
+        bypassCache: ev && (ev.ctrlKey || ev.metaKey || ev.shiftKey),
     });
 };
 
 /******************************************************************************/
 /******************************************************************************/
 
-(function() {
+(( ) => {
     const reRFC3986 = /^([^:\/?#]+:)?(\/\/[^\/?#]*)?([^?#]*)(\?[^#]*)?(#.*)?/;
     const reSchemeOnly = /^[\w-]+:$/;
     const staticFilterTypes = {
-        'beacon': 'other',
+        'beacon': 'ping',
         'doc': 'document',
         'css': 'stylesheet',
         'frame': 'subdocument',
-        'ping': 'other',
         'object_subrequest': 'object',
-        'xhr': 'xmlhttprequest'
+        'csp_report': 'other',
     };
     const createdStaticFilters = {};
 
@@ -1144,17 +1192,14 @@ const reloadTab = function(ev) {
         }
     };
 
-    const colorize = function() {
-        messaging.send(
-            'loggerUI',
-            {
-                what: 'getURLFilteringData',
-                context: selectValue('select.dynamic.origin'),
-                urls: targetURLs,
-                type: uglyTypeFromSelector('dynamic')
-            },
-            onColorsReady
-        );
+    const colorize = async function() {
+        const response = await messaging.send('loggerUI', {
+            what: 'getURLFilteringData',
+            context: selectValue('select.dynamic.origin'),
+            urls: targetURLs,
+            type: uglyTypeFromSelector('dynamic'),
+        });
+        onColorsReady(response);
     };
 
     const parseStaticInputs = function() {
@@ -1185,7 +1230,7 @@ const reloadTab = function(ev) {
         value = selectValue('select.static.origin');
         if ( value !== '' ) {
             if ( value === targetDomain ) {
-                options.push('first-party');
+                options.push('1p');
             } else {
                 options.push('domain=' + value);
             }
@@ -1208,55 +1253,62 @@ const reloadTab = function(ev) {
         );
     };
 
-    const onClick = function(ev) {
+    const onClick = async function(ev) {
         const target = ev.target;
         const tcl = target.classList;
 
         // Select a mode
         if ( tcl.contains('header') ) {
-            dialog.setAttribute('data-pane', target.getAttribute('data-pane') );
             ev.stopPropagation();
+            dialog.setAttribute('data-pane', target.getAttribute('data-pane') );
             return;
         }
 
+        // Toggle temporary exception filter
+        if ( tcl.contains('exceptor') ) {
+            ev.stopPropagation();
+            const status = await messaging.send('loggerUI', {
+                what: 'toggleTemporaryException',
+                filter: filterFromTargetRow(),
+            });
+            const row = target.closest('div');
+            row.classList.toggle('exceptored', status);
+            return;
+        }
+        
         // Create static filter
         if ( target.id === 'createStaticFilter' ) {
+            ev.stopPropagation();
             const value = staticFilterNode().value;
             // Avoid duplicates
-            if ( createdStaticFilters.hasOwnProperty(value) ) {
-                return;
-            }
+            if ( createdStaticFilters.hasOwnProperty(value) ) { return; }
             createdStaticFilters[value] = true;
+            // https://github.com/uBlockOrigin/uBlock-issues/issues/1281#issuecomment-704217175
+            // TODO:
+            //   Figure a way to use the actual document URL. Currently using
+            //   a synthetic URL derived from the document hostname.
             if ( value !== '' ) {
-                messaging.send(
-                    'loggerUI',
-                    {
-                        what: 'createUserFilter',
-                        autoComment: true,
-                        filters: value,
-                        origin: targetPageDomain,
-                        pageDomain: targetPageDomain,
-                    }
-                );
+                messaging.send('loggerUI', {
+                    what: 'createUserFilter',
+                    autoComment: true,
+                    filters: value,
+                    docURL: `https://${targetFrameHostname}/`,
+                });
             }
             updateWidgets();
-            ev.stopPropagation();
             return;
         }
 
         // Save url filtering rule(s)
         if ( target.id === 'saveRules' ) {
-                messaging.send(
-                'loggerUI',
-                {
-                    what: 'saveURLFilteringRules',
-                    context: selectValue('select.dynamic.origin'),
-                    urls: targetURLs,
-                    type: uglyTypeFromSelector('dynamic')
-                },
-                colorize
-            );
             ev.stopPropagation();
+            await messaging.send('loggerUI', {
+                what: 'saveURLFilteringRules',
+                context: selectValue('select.dynamic.origin'),
+                urls: targetURLs,
+                type: uglyTypeFromSelector('dynamic'),
+            });
+            colorize();
             return;
         }
 
@@ -1264,101 +1316,83 @@ const reloadTab = function(ev) {
 
         // Remove url filtering rule
         if ( tcl.contains('action') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'setURLFilteringRule',
-                    context: selectValue('select.dynamic.origin'),
-                    url: target.getAttribute('data-url'),
-                    type: uglyTypeFromSelector('dynamic'),
-                    action: 0,
-                    persist: persist
-                },
-                colorize
-            );
             ev.stopPropagation();
+            await messaging.send('loggerUI', {
+                what: 'setURLFilteringRule',
+                context: selectValue('select.dynamic.origin'),
+                url: target.getAttribute('data-url'),
+                type: uglyTypeFromSelector('dynamic'),
+                action: 0,
+                persist: persist,
+            });
+            colorize();
             return;
         }
 
         // add "allow" url filtering rule
         if ( tcl.contains('allow') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'setURLFilteringRule',
-                    context: selectValue('select.dynamic.origin'),
-                    url: target.parentNode.getAttribute('data-url'),
-                    type: uglyTypeFromSelector('dynamic'),
-                    action: 2,
-                    persist: persist
-                },
-                colorize
-            );
             ev.stopPropagation();
+            await messaging.send('loggerUI', {
+                what: 'setURLFilteringRule',
+                context: selectValue('select.dynamic.origin'),
+                url: target.parentNode.getAttribute('data-url'),
+                type: uglyTypeFromSelector('dynamic'),
+                action: 2,
+                persist: persist,
+            });
+            colorize();
             return;
         }
 
         // add "block" url filtering rule
         if ( tcl.contains('noop') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'setURLFilteringRule',
-                    context: selectValue('select.dynamic.origin'),
-                    url: target.parentNode.getAttribute('data-url'),
-                    type: uglyTypeFromSelector('dynamic'),
-                    action: 3,
-                    persist: persist
-                },
-                colorize
-            );
             ev.stopPropagation();
+            await messaging.send('loggerUI', {
+                what: 'setURLFilteringRule',
+                context: selectValue('select.dynamic.origin'),
+                url: target.parentNode.getAttribute('data-url'),
+                type: uglyTypeFromSelector('dynamic'),
+                action: 3,
+                persist: persist,
+            });
+            colorize();
             return;
         }
 
         // add "block" url filtering rule
         if ( tcl.contains('block') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'setURLFilteringRule',
-                    context: selectValue('select.dynamic.origin'),
-                    url: target.parentNode.getAttribute('data-url'),
-                    type: uglyTypeFromSelector('dynamic'),
-                    action: 1,
-                    persist: persist
-                },
-                colorize
-            );
             ev.stopPropagation();
+            await messaging.send('loggerUI', {
+                what: 'setURLFilteringRule',
+                context: selectValue('select.dynamic.origin'),
+                url: target.parentNode.getAttribute('data-url'),
+                type: uglyTypeFromSelector('dynamic'),
+                action: 1,
+                persist: persist,
+            });
+            colorize();
             return;
         }
 
         // Force a reload of the tab
         if ( tcl.contains('reload') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'reloadTab',
-                    tabId: targetTabId
-                }
-            );
             ev.stopPropagation();
+            messaging.send('loggerUI', {
+                what: 'reloadTab',
+                tabId: targetTabId,
+            });
             return;
         }
 
         // Hightlight corresponding element in target web page
         if ( tcl.contains('picker') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'launchElementPicker',
-                    tabId: targetTabId,
-                    targetURL: 'img\t' + targetURLs[0],
-                    select: true
-                }
-            );
             ev.stopPropagation();
+            messaging.send('loggerUI', {
+                what: 'launchElementPicker',
+                tabId: targetTabId,
+                targetURL: 'img\t' + targetURLs[0],
+                select: true,
+            });
             return;
         }
     };
@@ -1450,9 +1484,47 @@ const reloadTab = function(ev) {
         return urls;
     };
 
-    const fillSummaryPaneFilterList = function(rows) {
+    const filterFromTargetRow = function() {
+        return targetRow.children[1].textContent;
+    };
+
+    const aliasURLFromID = function(id) {
+        if ( id === '' ) { return ''; }
+        for ( const entry of loggerEntries ) {
+            if ( entry.id !== id || entry.aliased ) { continue; }
+            const fields = entry.textContent.split('\t');
+            return fields[6] || '';
+        }
+        return '';
+    };
+
+    const toSummaryPaneFilterNode = async function(receiver, filter) {
+        receiver.children[1].textContent = filter;
+        if ( filterAuthorMode !== true ) { return; }
+        const match = /#@?#/.exec(filter);
+        if ( match === null ) { return; }
+        const fragment = document.createDocumentFragment();
+        const pos = match.index + match[0].length;
+        fragment.appendChild(document.createTextNode(filter.slice(0, pos)));
+        const selector = filter.slice(pos);
+        const span = document.createElement('span');
+        span.className = 'filter';
+        span.textContent = selector;
+        fragment.appendChild(span);
+        const isTemporaryException = await messaging.send('loggerUI', {
+            what: 'hasTemporaryException',
+            filter,
+        });
+        receiver.classList.toggle('exceptored', isTemporaryException);
+        if ( match[0] === '##' || isTemporaryException ) {
+            receiver.children[2].style.visibility = '';
+        }
+        receiver.children[1].textContent = '';
+        receiver.children[1].appendChild(fragment);
+    };
+
+    const fillSummaryPaneFilterList = async function(rows) {
         const rawFilter = targetRow.children[1].textContent;
-        const compiledFilter = targetRow.getAttribute('data-filter');
 
         const nodeFromFilter = function(filter, lists) {
             const fragment = document.createDocumentFragment();
@@ -1492,7 +1564,7 @@ const reloadTab = function(ev) {
                 bestMatchFilter !== '' &&
                 Array.isArray(response[bestMatchFilter])
             ) {
-                rows[0].children[1].textContent = bestMatchFilter;
+                toSummaryPaneFilterNode(rows[0], bestMatchFilter);
                 rows[1].children[1].appendChild(nodeFromFilter(
                     bestMatchFilter,
                     response[bestMatchFilter]
@@ -1509,25 +1581,18 @@ const reloadTab = function(ev) {
         };
 
         if ( targetRow.classList.contains('networkRealm') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'listsFromNetFilter',
-                    compiledFilter: compiledFilter,
-                    rawFilter: rawFilter
-                },
-                handleResponse
-            );
-        } else if ( targetRow.classList.contains('cosmeticRealm') ) {
-            messaging.send(
-                'loggerUI',
-                {
-                    what: 'listsFromCosmeticFilter',
-                    url: targetRow.children[6].textContent,
-                    rawFilter: rawFilter,
-                },
-                handleResponse
-            );
+            const response = await messaging.send('loggerUI', {
+                what: 'listsFromNetFilter',
+                rawFilter: rawFilter,
+            });
+            handleResponse(response);
+        } else if ( targetRow.classList.contains('extendedRealm') ) {
+            const response = await messaging.send('loggerUI', {
+                what: 'listsFromCosmeticFilter',
+                url: targetRow.children[6].textContent,
+                rawFilter: rawFilter,
+            });
+            handleResponse(response);
         }
     };
 
@@ -1538,12 +1603,12 @@ const reloadTab = function(ev) {
         const trch = tr.children;
         let text;
         // Filter and context
-        text = trch[1].textContent;
+        text = filterFromTargetRow();
         if (
             (text !== '') &&
-            (trcl.contains('cosmeticRealm') || trcl.contains('networkRealm'))
+            (trcl.contains('extendedRealm') || trcl.contains('networkRealm'))
         ) {
-            rows[0].children[1].textContent = text;
+            toSummaryPaneFilterNode(rows[0], text);
         } else {
             rows[0].style.display = 'none';
         }
@@ -1553,7 +1618,7 @@ const reloadTab = function(ev) {
             (
                 trcl.contains('dynamicHost') ||
                 trcl.contains('dynamicUrl') ||
-                trcl.contains('switch')
+                trcl.contains('switchRealm')
             )
         ) {
             rows[2].children[1].textContent = text;
@@ -1594,21 +1659,38 @@ const reloadTab = function(ev) {
             rows[6].style.display = 'none';
         }
         // URL
-        text = trch[6].textContent;
-        if ( text !== '' ) {
+        const canonicalURL = trch[6].textContent;
+        if ( canonicalURL !== '' ) {
             const attr = tr.getAttribute('data-status') || '';
             if ( attr !== '' ) {
                 rows[7].setAttribute('data-status', attr);
+                if ( tr.hasAttribute('data-modifier') ) {
+                    rows[7].setAttribute('data-modifier', '');
+                }
             }
             rows[7].children[1].appendChild(trch[6].cloneNode(true));
         } else {
             rows[7].style.display = 'none';
         }
+        // Alias URL
+        text = tr.getAttribute('data-aliasid');
+        const aliasURL = text ? aliasURLFromID(text) : '';
+        if ( aliasURL !== '' ) {
+            rows[8].children[1].textContent =
+                vAPI.hostnameFromURI(aliasURL) + ' \u21d2\n\u2003' +
+                vAPI.hostnameFromURI(canonicalURL);
+            rows[9].children[1].textContent = aliasURL;
+        } else {
+            rows[8].style.display = 'none';
+            rows[9].style.display = 'none';
+        }
     };
 
     // Fill dynamic URL filtering pane
     const fillDynamicPane = function() {
-        if ( targetRow.classList.contains('cosmeticRealm') ) { return; }
+        if ( targetRow.classList.contains('extendedRealm') ) {
+            return;
+        }
 
         // https://github.com/uBlockOrigin/uBlock-issues/issues/662#issuecomment-509220702
         if ( targetType === 'doc' ) { return; }
@@ -1643,8 +1725,6 @@ const reloadTab = function(ev) {
         }
 
         colorize();
-
-        uDom('#modalOverlayContainer [data-pane="dynamic"]').removeClass('hide');
     };
 
     const fillOriginSelect = function(select, hostname, domain) {
@@ -1664,7 +1744,9 @@ const reloadTab = function(ev) {
 
     // Fill static filtering pane
     const fillStaticPane = function() {
-        if ( targetRow.classList.contains('cosmeticRealm') ) { return; }
+        if ( targetRow.classList.contains('extendedRealm') ) {
+            return;
+        }
 
         const template = vAPI.i18n('loggerStaticFilteringSentence');
         const rePlaceholder = /\{\{[^}]+?\}\}/g;
@@ -1697,12 +1779,13 @@ const reloadTab = function(ev) {
                 nodes.push(select);
                 break;
 
-            case '{{type}}':
+            case '{{type}}': {
+                const filterType = staticFilterTypes[targetType] || targetType;
                 select = document.createElement('select');
                 select.className = 'static type';
                 option = document.createElement('option');
-                option.setAttribute('value', targetType);
-                option.textContent = vAPI.i18n('loggerStaticFilteringSentencePartType').replace('{{type}}', targetType);
+                option.setAttribute('value', filterType);
+                option.textContent = vAPI.i18n('loggerStaticFilteringSentencePartType').replace('{{type}}', filterType);
                 select.appendChild(option);
                 option = document.createElement('option');
                 option.setAttribute('value', '');
@@ -1710,7 +1793,7 @@ const reloadTab = function(ev) {
                 select.appendChild(option);
                 nodes.push(select);
                 break;
-
+            }
             case '{{url}}':
                 select = document.createElement('select');
                 select.className = 'static url';
@@ -1773,8 +1856,8 @@ const reloadTab = function(ev) {
             }
         );
         dialog.classList.toggle(
-            'cosmeticRealm',
-            targetRow.classList.contains('cosmeticRealm')
+            'extendedRealm',
+            targetRow.classList.contains('extendedRealm')
         );
         targetDomain = domains[0];
         targetPageDomain = domains[1];
@@ -1783,13 +1866,13 @@ const reloadTab = function(ev) {
         fillSummaryPane();
         fillDynamicPane();
         fillStaticPane();
-        dialog.addEventListener('click', onClick, true);
+        dialog.addEventListener('click', ev => { onClick(ev); }, true);
         dialog.addEventListener('change', onSelectChange, true);
         dialog.addEventListener('input', onInputChange, true);
         modalDialog.show();
     };
 
-    const toggleOn = function(ev) {
+    const toggleOn = async function(ev) {
         targetRow = ev.target.closest('.canDetails');
         if ( targetRow === null ) { return; }
         ev.stopPropagation();
@@ -1800,29 +1883,24 @@ const reloadTab = function(ev) {
         targetFrameHostname = targetRow.getAttribute('data-dochn') || '';
 
         // We need the root domain names for best user experience.
-        messaging.send(
-            'loggerUI',
-            {
-                what: 'getDomainNames',
-                targets: [
-                    targetURLs[0],
-                    targetPageHostname,
-                    targetFrameHostname
-                ]
-            },
-            fillDialog
-        );
+        const domains = await messaging.send('loggerUI', {
+            what: 'getDomainNames',
+            targets: [
+                targetURLs[0],
+                targetPageHostname,
+                targetFrameHostname
+            ],
+        });
+        fillDialog(domains);
     };
 
     uDom('#netInspector').on(
         'click',
         '.canDetails > span:nth-of-type(2),.canDetails > span:nth-of-type(3),.canDetails > span:nth-of-type(5)',
-        toggleOn
+        ev => { toggleOn(ev); }
     );
 })();
 
-// https://www.youtube.com/watch?v=XyNYrmmdUd4
-
 /******************************************************************************/
 /******************************************************************************/
 
@@ -1950,14 +2028,14 @@ const rowFilterer = (( ) => {
         );
     };
 
-    const onFilterChangedAsync = (function() {
+    const onFilterChangedAsync = (( ) => {
         let timer;
         const commit = ( ) => {
             timer = undefined;
             parseInput();
             filterAll();
         };
-        return function() {
+        return ( ) => {
             if ( timer !== undefined ) {
                 clearTimeout(timer);
             }
@@ -2272,7 +2350,7 @@ const popupManager = (( ) => {
 
     const setTabId = function(tabId) {
         if ( popup === null ) { return; }
-        popup.setAttribute('src', 'popup.html?tabId=' + tabId);
+        popup.setAttribute('src', 'popup-fenix.html?portrait=1&tabId=' + tabId);
     };
 
     const onTabIdChanged = function() {
@@ -2339,10 +2417,10 @@ const popupManager = (( ) => {
 // Filter hit stats' MVP ("minimum viable product")
 //
 const loggerStats = (( ) => {
+    const enabled = false;
     const filterHits = new Map();
     let dialog;
     let timer;
-
     const makeRow = function() {
         const div = document.createElement('div');
         div.appendChild(document.createElement('span'));
@@ -2406,6 +2484,7 @@ const loggerStats = (( ) => {
 
     return {
         processFilter: function(filter) {
+            if ( enabled !== true ) { return; }
             if ( filter.source !== 'static' && filter.source !== 'cosmetic' ) {
                 return;
             }
@@ -2624,9 +2703,9 @@ const loggerSettings = (( ) => {
         linesPerEntry: 4,
     };
 
-    {
+    vAPI.localStorage.getItemAsync('loggerSettings').then(value => {
         try {
-            const stored = JSON.parse(vAPI.localStorage.getItem('loggerSettings'));
+            const stored = JSON.parse(value);
             if ( typeof stored.discard.maxAge === 'number' ) {
                 settings.discard.maxAge = stored.discard.maxAge;
             }
@@ -2644,7 +2723,7 @@ const loggerSettings = (( ) => {
             }
         } catch(ex) {
         }
-    }
+    });
 
     const valueFromInput = function(input, def) {
         let value = parseInt(input.value, 10);
@@ -2736,7 +2815,7 @@ logger.resize = (function() {
             const crect = elem.getBoundingClientRect();
             const dh = crect.bottom - vrect.bottom;
             if ( dh === 0 ) { continue; }
-            elem.style.height = (crect.height - dh) + 'px';
+            elem.style.height = Math.ceil(crect.height - dh) + 'px';
         }
     };
 
@@ -2766,10 +2845,10 @@ const grabView = function() {
 
 const releaseView = function() {
     if ( logger.ownerId === undefined ) { return; }
-    vAPI.messaging.send(
-        'loggerUI',
-        { what: 'releaseView', ownerId: logger.ownerId }
-    );
+    vAPI.messaging.send('loggerUI', {
+        what: 'releaseView',
+        ownerId: logger.ownerId,
+    });
     logger.ownerId = undefined;
 };
 
